/****************************************************************************
** Meta object code from reading C++ file 'SetupDialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../lmms/include/SetupDialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SetupDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SetupDialog_t {
    QByteArrayData data[66];
    char stringdata0[918];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SetupDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SetupDialog_t qt_meta_stringdata_SetupDialog = {
    {
QT_MOC_LITERAL(0, 0, 11), // "SetupDialog"
QT_MOC_LITERAL(1, 12, 6), // "accept"
QT_MOC_LITERAL(2, 19, 0), // ""
QT_MOC_LITERAL(3, 20, 13), // "setBufferSize"
QT_MOC_LITERAL(4, 34, 6), // "_value"
QT_MOC_LITERAL(5, 41, 12), // "resetBufSize"
QT_MOC_LITERAL(6, 54, 18), // "displayBufSizeHelp"
QT_MOC_LITERAL(7, 73, 13), // "setWorkingDir"
QT_MOC_LITERAL(8, 87, 3), // "_wd"
QT_MOC_LITERAL(9, 91, 9), // "setVSTDir"
QT_MOC_LITERAL(10, 101, 3), // "_vd"
QT_MOC_LITERAL(11, 105, 9), // "setGIGDir"
QT_MOC_LITERAL(12, 115, 3), // "_gd"
QT_MOC_LITERAL(13, 119, 9), // "setSF2Dir"
QT_MOC_LITERAL(14, 129, 4), // "_sfd"
QT_MOC_LITERAL(15, 134, 13), // "setArtworkDir"
QT_MOC_LITERAL(16, 148, 3), // "_ad"
QT_MOC_LITERAL(17, 152, 12), // "setLADSPADir"
QT_MOC_LITERAL(18, 165, 3), // "_ld"
QT_MOC_LITERAL(19, 169, 9), // "setSTKDir"
QT_MOC_LITERAL(20, 179, 3), // "_sd"
QT_MOC_LITERAL(21, 183, 19), // "setDefaultSoundfont"
QT_MOC_LITERAL(22, 203, 3), // "_sf"
QT_MOC_LITERAL(23, 207, 20), // "setBackgroundArtwork"
QT_MOC_LITERAL(24, 228, 3), // "_ba"
QT_MOC_LITERAL(25, 232, 19), // "setAutoSaveInterval"
QT_MOC_LITERAL(26, 252, 4), // "time"
QT_MOC_LITERAL(27, 257, 13), // "resetAutoSave"
QT_MOC_LITERAL(28, 271, 23), // "displaySaveIntervalHelp"
QT_MOC_LITERAL(29, 295, 21), // "audioInterfaceChanged"
QT_MOC_LITERAL(30, 317, 7), // "_driver"
QT_MOC_LITERAL(31, 325, 16), // "displayAudioHelp"
QT_MOC_LITERAL(32, 342, 20), // "midiInterfaceChanged"
QT_MOC_LITERAL(33, 363, 15), // "displayMIDIHelp"
QT_MOC_LITERAL(34, 379, 14), // "toggleToolTips"
QT_MOC_LITERAL(35, 394, 8), // "_enabled"
QT_MOC_LITERAL(36, 403, 20), // "toggleWarnAfterSetup"
QT_MOC_LITERAL(37, 424, 17), // "toggleDisplaydBFS"
QT_MOC_LITERAL(38, 442, 10), // "toggleMMPZ"
QT_MOC_LITERAL(39, 453, 19), // "toggleDisableBackup"
QT_MOC_LITERAL(40, 473, 21), // "toggleOpenLastProject"
QT_MOC_LITERAL(41, 495, 16), // "toggleHQAudioDev"
QT_MOC_LITERAL(42, 512, 14), // "openWorkingDir"
QT_MOC_LITERAL(43, 527, 10), // "openVSTDir"
QT_MOC_LITERAL(44, 538, 10), // "openGIGDir"
QT_MOC_LITERAL(45, 549, 10), // "openSF2Dir"
QT_MOC_LITERAL(46, 560, 14), // "openArtworkDir"
QT_MOC_LITERAL(47, 575, 13), // "openLADSPADir"
QT_MOC_LITERAL(48, 589, 10), // "openSTKDir"
QT_MOC_LITERAL(49, 600, 20), // "openDefaultSoundfont"
QT_MOC_LITERAL(50, 621, 21), // "openBackgroundArtwork"
QT_MOC_LITERAL(51, 643, 18), // "toggleSmoothScroll"
QT_MOC_LITERAL(52, 662, 14), // "toggleAutoSave"
QT_MOC_LITERAL(53, 677, 21), // "toggleRunningAutoSave"
QT_MOC_LITERAL(54, 699, 30), // "toggleOneInstrumentTrackWindow"
QT_MOC_LITERAL(55, 730, 25), // "toggleCompactTrackButtons"
QT_MOC_LITERAL(56, 756, 20), // "toggleSyncVSTPlugins"
QT_MOC_LITERAL(57, 777, 16), // "toggleAnimateAFP"
QT_MOC_LITERAL(58, 794, 16), // "toggleNoteLabels"
QT_MOC_LITERAL(59, 811, 2), // "en"
QT_MOC_LITERAL(60, 814, 21), // "toggleDisplayWaveform"
QT_MOC_LITERAL(61, 836, 21), // "toggleDisableAutoquit"
QT_MOC_LITERAL(62, 858, 21), // "vstEmbedMethodChanged"
QT_MOC_LITERAL(63, 880, 20), // "toggleVSTAlwaysOnTop"
QT_MOC_LITERAL(64, 901, 11), // "setLanguage"
QT_MOC_LITERAL(65, 913, 4) // "lang"

    },
    "SetupDialog\0accept\0\0setBufferSize\0"
    "_value\0resetBufSize\0displayBufSizeHelp\0"
    "setWorkingDir\0_wd\0setVSTDir\0_vd\0"
    "setGIGDir\0_gd\0setSF2Dir\0_sfd\0setArtworkDir\0"
    "_ad\0setLADSPADir\0_ld\0setSTKDir\0_sd\0"
    "setDefaultSoundfont\0_sf\0setBackgroundArtwork\0"
    "_ba\0setAutoSaveInterval\0time\0resetAutoSave\0"
    "displaySaveIntervalHelp\0audioInterfaceChanged\0"
    "_driver\0displayAudioHelp\0midiInterfaceChanged\0"
    "displayMIDIHelp\0toggleToolTips\0_enabled\0"
    "toggleWarnAfterSetup\0toggleDisplaydBFS\0"
    "toggleMMPZ\0toggleDisableBackup\0"
    "toggleOpenLastProject\0toggleHQAudioDev\0"
    "openWorkingDir\0openVSTDir\0openGIGDir\0"
    "openSF2Dir\0openArtworkDir\0openLADSPADir\0"
    "openSTKDir\0openDefaultSoundfont\0"
    "openBackgroundArtwork\0toggleSmoothScroll\0"
    "toggleAutoSave\0toggleRunningAutoSave\0"
    "toggleOneInstrumentTrackWindow\0"
    "toggleCompactTrackButtons\0"
    "toggleSyncVSTPlugins\0toggleAnimateAFP\0"
    "toggleNoteLabels\0en\0toggleDisplayWaveform\0"
    "toggleDisableAutoquit\0vstEmbedMethodChanged\0"
    "toggleVSTAlwaysOnTop\0setLanguage\0lang"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SetupDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      49,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  259,    2, 0x09 /* Protected */,
       3,    1,  260,    2, 0x08 /* Private */,
       5,    0,  263,    2, 0x08 /* Private */,
       6,    0,  264,    2, 0x08 /* Private */,
       7,    1,  265,    2, 0x08 /* Private */,
       9,    1,  268,    2, 0x08 /* Private */,
      11,    1,  271,    2, 0x08 /* Private */,
      13,    1,  274,    2, 0x08 /* Private */,
      15,    1,  277,    2, 0x08 /* Private */,
      17,    1,  280,    2, 0x08 /* Private */,
      19,    1,  283,    2, 0x08 /* Private */,
      21,    1,  286,    2, 0x08 /* Private */,
      23,    1,  289,    2, 0x08 /* Private */,
      25,    1,  292,    2, 0x08 /* Private */,
      27,    0,  295,    2, 0x08 /* Private */,
      28,    0,  296,    2, 0x08 /* Private */,
      29,    1,  297,    2, 0x08 /* Private */,
      31,    0,  300,    2, 0x08 /* Private */,
      32,    1,  301,    2, 0x08 /* Private */,
      33,    0,  304,    2, 0x08 /* Private */,
      34,    1,  305,    2, 0x08 /* Private */,
      36,    1,  308,    2, 0x08 /* Private */,
      37,    1,  311,    2, 0x08 /* Private */,
      38,    1,  314,    2, 0x08 /* Private */,
      39,    1,  317,    2, 0x08 /* Private */,
      40,    1,  320,    2, 0x08 /* Private */,
      41,    1,  323,    2, 0x08 /* Private */,
      42,    0,  326,    2, 0x08 /* Private */,
      43,    0,  327,    2, 0x08 /* Private */,
      44,    0,  328,    2, 0x08 /* Private */,
      45,    0,  329,    2, 0x08 /* Private */,
      46,    0,  330,    2, 0x08 /* Private */,
      47,    0,  331,    2, 0x08 /* Private */,
      48,    0,  332,    2, 0x08 /* Private */,
      49,    0,  333,    2, 0x08 /* Private */,
      50,    0,  334,    2, 0x08 /* Private */,
      51,    1,  335,    2, 0x08 /* Private */,
      52,    1,  338,    2, 0x08 /* Private */,
      53,    1,  341,    2, 0x08 /* Private */,
      54,    1,  344,    2, 0x08 /* Private */,
      55,    1,  347,    2, 0x08 /* Private */,
      56,    1,  350,    2, 0x08 /* Private */,
      57,    1,  353,    2, 0x08 /* Private */,
      58,    1,  356,    2, 0x08 /* Private */,
      60,    1,  359,    2, 0x08 /* Private */,
      61,    1,  362,    2, 0x08 /* Private */,
      62,    0,  365,    2, 0x08 /* Private */,
      63,    1,  366,    2, 0x08 /* Private */,
      64,    1,  369,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void, QMetaType::QString,   14,
    QMetaType::Void, QMetaType::QString,   16,
    QMetaType::Void, QMetaType::QString,   18,
    QMetaType::Void, QMetaType::QString,   20,
    QMetaType::Void, QMetaType::QString,   22,
    QMetaType::Void, QMetaType::QString,   24,
    QMetaType::Void, QMetaType::Int,   26,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   30,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   30,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   35,
    QMetaType::Void, QMetaType::Bool,   35,
    QMetaType::Void, QMetaType::Bool,   35,
    QMetaType::Void, QMetaType::Bool,   35,
    QMetaType::Void, QMetaType::Bool,   35,
    QMetaType::Void, QMetaType::Bool,   35,
    QMetaType::Void, QMetaType::Bool,   35,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   35,
    QMetaType::Void, QMetaType::Bool,   35,
    QMetaType::Void, QMetaType::Bool,   35,
    QMetaType::Void, QMetaType::Bool,   35,
    QMetaType::Void, QMetaType::Bool,   35,
    QMetaType::Void, QMetaType::Bool,   35,
    QMetaType::Void, QMetaType::Bool,   35,
    QMetaType::Void, QMetaType::Bool,   59,
    QMetaType::Void, QMetaType::Bool,   59,
    QMetaType::Void, QMetaType::Bool,   59,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   59,
    QMetaType::Void, QMetaType::Int,   65,

       0        // eod
};

void SetupDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SetupDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->accept(); break;
        case 1: _t->setBufferSize((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->resetBufSize(); break;
        case 3: _t->displayBufSizeHelp(); break;
        case 4: _t->setWorkingDir((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->setVSTDir((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->setGIGDir((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->setSF2Dir((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->setArtworkDir((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 9: _t->setLADSPADir((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->setSTKDir((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->setDefaultSoundfont((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 12: _t->setBackgroundArtwork((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 13: _t->setAutoSaveInterval((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->resetAutoSave(); break;
        case 15: _t->displaySaveIntervalHelp(); break;
        case 16: _t->audioInterfaceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 17: _t->displayAudioHelp(); break;
        case 18: _t->midiInterfaceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 19: _t->displayMIDIHelp(); break;
        case 20: _t->toggleToolTips((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 21: _t->toggleWarnAfterSetup((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 22: _t->toggleDisplaydBFS((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 23: _t->toggleMMPZ((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 24: _t->toggleDisableBackup((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 25: _t->toggleOpenLastProject((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 26: _t->toggleHQAudioDev((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 27: _t->openWorkingDir(); break;
        case 28: _t->openVSTDir(); break;
        case 29: _t->openGIGDir(); break;
        case 30: _t->openSF2Dir(); break;
        case 31: _t->openArtworkDir(); break;
        case 32: _t->openLADSPADir(); break;
        case 33: _t->openSTKDir(); break;
        case 34: _t->openDefaultSoundfont(); break;
        case 35: _t->openBackgroundArtwork(); break;
        case 36: _t->toggleSmoothScroll((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 37: _t->toggleAutoSave((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 38: _t->toggleRunningAutoSave((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 39: _t->toggleOneInstrumentTrackWindow((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 40: _t->toggleCompactTrackButtons((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 41: _t->toggleSyncVSTPlugins((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 42: _t->toggleAnimateAFP((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 43: _t->toggleNoteLabels((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 44: _t->toggleDisplayWaveform((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 45: _t->toggleDisableAutoquit((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 46: _t->vstEmbedMethodChanged(); break;
        case 47: _t->toggleVSTAlwaysOnTop((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 48: _t->setLanguage((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SetupDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_SetupDialog.data,
    qt_meta_data_SetupDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SetupDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SetupDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SetupDialog.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int SetupDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 49)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 49;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 49)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 49;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
