/****************************************************************************
** Meta object code from reading C++ file 'Song.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../lmms/include/Song.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Song.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Song_t {
    QByteArrayData data[46];
    char stringdata0[591];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Song_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Song_t qt_meta_stringdata_Song = {
    {
QT_MOC_LITERAL(0, 0, 4), // "Song"
QT_MOC_LITERAL(1, 5, 13), // "projectLoaded"
QT_MOC_LITERAL(2, 19, 0), // ""
QT_MOC_LITERAL(3, 20, 20), // "playbackStateChanged"
QT_MOC_LITERAL(4, 41, 23), // "playbackPositionChanged"
QT_MOC_LITERAL(5, 65, 13), // "lengthChanged"
QT_MOC_LITERAL(6, 79, 5), // "tacts"
QT_MOC_LITERAL(7, 85, 12), // "tempoChanged"
QT_MOC_LITERAL(8, 98, 5), // "bpm_t"
QT_MOC_LITERAL(9, 104, 6), // "newBPM"
QT_MOC_LITERAL(10, 111, 20), // "timeSignatureChanged"
QT_MOC_LITERAL(11, 132, 15), // "oldTicksPerTact"
QT_MOC_LITERAL(12, 148, 12), // "ticksPerTact"
QT_MOC_LITERAL(13, 161, 15), // "controllerAdded"
QT_MOC_LITERAL(14, 177, 11), // "Controller*"
QT_MOC_LITERAL(15, 189, 17), // "controllerRemoved"
QT_MOC_LITERAL(16, 207, 18), // "updateSampleTracks"
QT_MOC_LITERAL(17, 226, 8), // "playSong"
QT_MOC_LITERAL(18, 235, 6), // "record"
QT_MOC_LITERAL(19, 242, 13), // "playAndRecord"
QT_MOC_LITERAL(20, 256, 6), // "playBB"
QT_MOC_LITERAL(21, 263, 11), // "playPattern"
QT_MOC_LITERAL(22, 275, 14), // "const Pattern*"
QT_MOC_LITERAL(23, 290, 13), // "patternToPlay"
QT_MOC_LITERAL(24, 304, 4), // "loop"
QT_MOC_LITERAL(25, 309, 11), // "togglePause"
QT_MOC_LITERAL(26, 321, 4), // "stop"
QT_MOC_LITERAL(27, 326, 13), // "importProject"
QT_MOC_LITERAL(28, 340, 13), // "exportProject"
QT_MOC_LITERAL(29, 354, 11), // "multiExport"
QT_MOC_LITERAL(30, 366, 19), // "exportProjectTracks"
QT_MOC_LITERAL(31, 386, 17), // "exportProjectMidi"
QT_MOC_LITERAL(32, 404, 11), // "startExport"
QT_MOC_LITERAL(33, 416, 10), // "stopExport"
QT_MOC_LITERAL(34, 427, 11), // "setModified"
QT_MOC_LITERAL(35, 439, 12), // "clearProject"
QT_MOC_LITERAL(36, 452, 10), // "addBBTrack"
QT_MOC_LITERAL(37, 463, 9), // "insertBar"
QT_MOC_LITERAL(38, 473, 9), // "removeBar"
QT_MOC_LITERAL(39, 483, 14), // "addSampleTrack"
QT_MOC_LITERAL(40, 498, 18), // "addAutomationTrack"
QT_MOC_LITERAL(41, 517, 8), // "setTempo"
QT_MOC_LITERAL(42, 526, 16), // "setTimeSignature"
QT_MOC_LITERAL(43, 543, 19), // "masterVolumeChanged"
QT_MOC_LITERAL(44, 563, 7), // "savePos"
QT_MOC_LITERAL(45, 571, 19) // "updateFramesPerTick"

    },
    "Song\0projectLoaded\0\0playbackStateChanged\0"
    "playbackPositionChanged\0lengthChanged\0"
    "tacts\0tempoChanged\0bpm_t\0newBPM\0"
    "timeSignatureChanged\0oldTicksPerTact\0"
    "ticksPerTact\0controllerAdded\0Controller*\0"
    "controllerRemoved\0updateSampleTracks\0"
    "playSong\0record\0playAndRecord\0playBB\0"
    "playPattern\0const Pattern*\0patternToPlay\0"
    "loop\0togglePause\0stop\0importProject\0"
    "exportProject\0multiExport\0exportProjectTracks\0"
    "exportProjectMidi\0startExport\0stopExport\0"
    "setModified\0clearProject\0addBBTrack\0"
    "insertBar\0removeBar\0addSampleTrack\0"
    "addAutomationTrack\0setTempo\0"
    "setTimeSignature\0masterVolumeChanged\0"
    "savePos\0updateFramesPerTick"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Song[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      36,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  194,    2, 0x06 /* Public */,
       3,    0,  195,    2, 0x06 /* Public */,
       4,    0,  196,    2, 0x06 /* Public */,
       5,    1,  197,    2, 0x06 /* Public */,
       7,    1,  200,    2, 0x06 /* Public */,
      10,    2,  203,    2, 0x06 /* Public */,
      13,    1,  208,    2, 0x06 /* Public */,
      15,    1,  211,    2, 0x06 /* Public */,
      16,    0,  214,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      17,    0,  215,    2, 0x0a /* Public */,
      18,    0,  216,    2, 0x0a /* Public */,
      19,    0,  217,    2, 0x0a /* Public */,
      20,    0,  218,    2, 0x0a /* Public */,
      21,    2,  219,    2, 0x0a /* Public */,
      21,    1,  224,    2, 0x2a /* Public | MethodCloned */,
      25,    0,  227,    2, 0x0a /* Public */,
      26,    0,  228,    2, 0x0a /* Public */,
      27,    0,  229,    2, 0x0a /* Public */,
      28,    1,  230,    2, 0x0a /* Public */,
      28,    0,  233,    2, 0x2a /* Public | MethodCloned */,
      30,    0,  234,    2, 0x0a /* Public */,
      31,    0,  235,    2, 0x0a /* Public */,
      32,    0,  236,    2, 0x0a /* Public */,
      33,    0,  237,    2, 0x0a /* Public */,
      34,    0,  238,    2, 0x0a /* Public */,
      35,    0,  239,    2, 0x0a /* Public */,
      36,    0,  240,    2, 0x0a /* Public */,
      37,    0,  241,    2, 0x08 /* Private */,
      38,    0,  242,    2, 0x08 /* Private */,
      39,    0,  243,    2, 0x08 /* Private */,
      40,    0,  244,    2, 0x08 /* Private */,
      41,    0,  245,    2, 0x08 /* Private */,
      42,    0,  246,    2, 0x08 /* Private */,
      43,    0,  247,    2, 0x08 /* Private */,
      44,    0,  248,    2, 0x08 /* Private */,
      45,    0,  249,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   11,   12,
    QMetaType::Void, 0x80000000 | 14,    2,
    QMetaType::Void, 0x80000000 | 14,    2,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 22, QMetaType::Bool,   23,   24,
    QMetaType::Void, 0x80000000 | 22,   23,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   29,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Song::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Song *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->projectLoaded(); break;
        case 1: _t->playbackStateChanged(); break;
        case 2: _t->playbackPositionChanged(); break;
        case 3: _t->lengthChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->tempoChanged((*reinterpret_cast< bpm_t(*)>(_a[1]))); break;
        case 5: _t->timeSignatureChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 6: _t->controllerAdded((*reinterpret_cast< Controller*(*)>(_a[1]))); break;
        case 7: _t->controllerRemoved((*reinterpret_cast< Controller*(*)>(_a[1]))); break;
        case 8: _t->updateSampleTracks(); break;
        case 9: _t->playSong(); break;
        case 10: _t->record(); break;
        case 11: _t->playAndRecord(); break;
        case 12: _t->playBB(); break;
        case 13: _t->playPattern((*reinterpret_cast< const Pattern*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 14: _t->playPattern((*reinterpret_cast< const Pattern*(*)>(_a[1]))); break;
        case 15: _t->togglePause(); break;
        case 16: _t->stop(); break;
        case 17: _t->importProject(); break;
        case 18: _t->exportProject((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 19: _t->exportProject(); break;
        case 20: _t->exportProjectTracks(); break;
        case 21: _t->exportProjectMidi(); break;
        case 22: _t->startExport(); break;
        case 23: _t->stopExport(); break;
        case 24: _t->setModified(); break;
        case 25: _t->clearProject(); break;
        case 26: _t->addBBTrack(); break;
        case 27: _t->insertBar(); break;
        case 28: _t->removeBar(); break;
        case 29: _t->addSampleTrack(); break;
        case 30: _t->addAutomationTrack(); break;
        case 31: _t->setTempo(); break;
        case 32: _t->setTimeSignature(); break;
        case 33: _t->masterVolumeChanged(); break;
        case 34: _t->savePos(); break;
        case 35: _t->updateFramesPerTick(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Controller* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Controller* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Song::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Song::projectLoaded)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Song::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Song::playbackStateChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Song::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Song::playbackPositionChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Song::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Song::lengthChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Song::*)(bpm_t );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Song::tempoChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (Song::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Song::timeSignatureChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (Song::*)(Controller * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Song::controllerAdded)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (Song::*)(Controller * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Song::controllerRemoved)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (Song::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Song::updateSampleTracks)) {
                *result = 8;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Song::staticMetaObject = { {
    QMetaObject::SuperData::link<TrackContainer::staticMetaObject>(),
    qt_meta_stringdata_Song.data,
    qt_meta_data_Song,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Song::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Song::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Song.stringdata0))
        return static_cast<void*>(this);
    return TrackContainer::qt_metacast(_clname);
}

int Song::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TrackContainer::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    return _id;
}

// SIGNAL 0
void Song::projectLoaded()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Song::playbackStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Song::playbackPositionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Song::lengthChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Song::tempoChanged(bpm_t _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void Song::timeSignatureChanged(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void Song::controllerAdded(Controller * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void Song::controllerRemoved(Controller * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void Song::updateSampleTracks()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
