/********************************************************************************
** Form generated from reading UI file 'export_project.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXPORT_PROJECT_H
#define UI_EXPORT_PROJECT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ExportProjectDialog
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QGroupBox *outputGroupBox;
    QVBoxLayout *vboxLayout1;
    QLabel *label;
    QComboBox *fileFormatCB;
    QWidget *sampleRateWidget;
    QVBoxLayout *verticalLayout;
    QLabel *labelSampleRate;
    QComboBox *samplerateCB;
    QWidget *depthWidget;
    QVBoxLayout *vboxLayout2;
    QLabel *labelBitDepth;
    QComboBox *depthCB;
    QWidget *stereoModeWidget;
    QVBoxLayout *verticalLayout_4;
    QLabel *labelStereoMode_3;
    QComboBox *stereoModeComboBox;
    QWidget *bitrateWidget;
    QVBoxLayout *vboxLayout3;
    QLabel *labelBitRate;
    QComboBox *bitrateCB;
    QCheckBox *checkBoxVariableBitRate;
    QSpacerItem *spacerItem;
    QGroupBox *qualityGroupBox;
    QVBoxLayout *vboxLayout4;
    QLabel *label_2;
    QComboBox *interpolationCB;
    QLabel *label_4;
    QComboBox *oversamplingCB;
    QCheckBox *exportLoopCB;
    QCheckBox *renderMarkersCB;
    QSpacerItem *spacerItem1;
    QHBoxLayout *hboxLayout1;
    QSpacerItem *spacerItem2;
    QPushButton *startButton;
    QPushButton *cancelButton;
    QProgressBar *progressBar;

    void setupUi(QDialog *ExportProjectDialog)
    {
        if (ExportProjectDialog->objectName().isEmpty())
            ExportProjectDialog->setObjectName(QString::fromUtf8("ExportProjectDialog"));
        ExportProjectDialog->resize(715, 491);
        ExportProjectDialog->setMinimumSize(QSize(519, 412));
        vboxLayout = new QVBoxLayout(ExportProjectDialog);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        outputGroupBox = new QGroupBox(ExportProjectDialog);
        outputGroupBox->setObjectName(QString::fromUtf8("outputGroupBox"));
        vboxLayout1 = new QVBoxLayout(outputGroupBox);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        vboxLayout1->setContentsMargins(9, -1, -1, -1);
        label = new QLabel(outputGroupBox);
        label->setObjectName(QString::fromUtf8("label"));

        vboxLayout1->addWidget(label);

        fileFormatCB = new QComboBox(outputGroupBox);
        fileFormatCB->setObjectName(QString::fromUtf8("fileFormatCB"));

        vboxLayout1->addWidget(fileFormatCB);

        sampleRateWidget = new QWidget(outputGroupBox);
        sampleRateWidget->setObjectName(QString::fromUtf8("sampleRateWidget"));
        verticalLayout = new QVBoxLayout(sampleRateWidget);
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        labelSampleRate = new QLabel(sampleRateWidget);
        labelSampleRate->setObjectName(QString::fromUtf8("labelSampleRate"));

        verticalLayout->addWidget(labelSampleRate);

        samplerateCB = new QComboBox(sampleRateWidget);
        samplerateCB->addItem(QString());
        samplerateCB->addItem(QString());
        samplerateCB->addItem(QString());
        samplerateCB->addItem(QString());
        samplerateCB->addItem(QString());
        samplerateCB->setObjectName(QString::fromUtf8("samplerateCB"));

        verticalLayout->addWidget(samplerateCB);


        vboxLayout1->addWidget(sampleRateWidget);

        depthWidget = new QWidget(outputGroupBox);
        depthWidget->setObjectName(QString::fromUtf8("depthWidget"));
        depthWidget->setMinimumSize(QSize(0, 20));
        vboxLayout2 = new QVBoxLayout(depthWidget);
        vboxLayout2->setContentsMargins(0, 0, 0, 0);
        vboxLayout2->setObjectName(QString::fromUtf8("vboxLayout2"));
        labelBitDepth = new QLabel(depthWidget);
        labelBitDepth->setObjectName(QString::fromUtf8("labelBitDepth"));

        vboxLayout2->addWidget(labelBitDepth);

        depthCB = new QComboBox(depthWidget);
        depthCB->addItem(QString());
        depthCB->addItem(QString());
        depthCB->addItem(QString());
        depthCB->setObjectName(QString::fromUtf8("depthCB"));

        vboxLayout2->addWidget(depthCB);


        vboxLayout1->addWidget(depthWidget);

        stereoModeWidget = new QWidget(outputGroupBox);
        stereoModeWidget->setObjectName(QString::fromUtf8("stereoModeWidget"));
        verticalLayout_4 = new QVBoxLayout(stereoModeWidget);
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        labelStereoMode_3 = new QLabel(stereoModeWidget);
        labelStereoMode_3->setObjectName(QString::fromUtf8("labelStereoMode_3"));

        verticalLayout_4->addWidget(labelStereoMode_3);

        stereoModeComboBox = new QComboBox(stereoModeWidget);
        stereoModeComboBox->addItem(QString());
        stereoModeComboBox->addItem(QString());
        stereoModeComboBox->addItem(QString());
        stereoModeComboBox->setObjectName(QString::fromUtf8("stereoModeComboBox"));

        verticalLayout_4->addWidget(stereoModeComboBox);


        vboxLayout1->addWidget(stereoModeWidget);

        bitrateWidget = new QWidget(outputGroupBox);
        bitrateWidget->setObjectName(QString::fromUtf8("bitrateWidget"));
        vboxLayout3 = new QVBoxLayout(bitrateWidget);
#ifndef Q_OS_MAC
        vboxLayout3->setSpacing(6);
#endif
        vboxLayout3->setContentsMargins(0, 0, 0, 0);
        vboxLayout3->setObjectName(QString::fromUtf8("vboxLayout3"));
        labelBitRate = new QLabel(bitrateWidget);
        labelBitRate->setObjectName(QString::fromUtf8("labelBitRate"));

        vboxLayout3->addWidget(labelBitRate);

        bitrateCB = new QComboBox(bitrateWidget);
        bitrateCB->addItem(QString());
        bitrateCB->addItem(QString());
        bitrateCB->addItem(QString());
        bitrateCB->addItem(QString());
        bitrateCB->addItem(QString());
        bitrateCB->addItem(QString());
        bitrateCB->setObjectName(QString::fromUtf8("bitrateCB"));

        vboxLayout3->addWidget(bitrateCB);

        checkBoxVariableBitRate = new QCheckBox(bitrateWidget);
        checkBoxVariableBitRate->setObjectName(QString::fromUtf8("checkBoxVariableBitRate"));

        vboxLayout3->addWidget(checkBoxVariableBitRate);


        vboxLayout1->addWidget(bitrateWidget);

        spacerItem = new QSpacerItem(163, 20, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout1->addItem(spacerItem);


        hboxLayout->addWidget(outputGroupBox);

        qualityGroupBox = new QGroupBox(ExportProjectDialog);
        qualityGroupBox->setObjectName(QString::fromUtf8("qualityGroupBox"));
        vboxLayout4 = new QVBoxLayout(qualityGroupBox);
        vboxLayout4->setObjectName(QString::fromUtf8("vboxLayout4"));
        label_2 = new QLabel(qualityGroupBox);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        vboxLayout4->addWidget(label_2);

        interpolationCB = new QComboBox(qualityGroupBox);
        interpolationCB->addItem(QString());
        interpolationCB->addItem(QString());
        interpolationCB->addItem(QString());
        interpolationCB->addItem(QString());
        interpolationCB->setObjectName(QString::fromUtf8("interpolationCB"));

        vboxLayout4->addWidget(interpolationCB);

        label_4 = new QLabel(qualityGroupBox);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        vboxLayout4->addWidget(label_4);

        oversamplingCB = new QComboBox(qualityGroupBox);
        oversamplingCB->addItem(QString());
        oversamplingCB->addItem(QString());
        oversamplingCB->addItem(QString());
        oversamplingCB->addItem(QString());
        oversamplingCB->setObjectName(QString::fromUtf8("oversamplingCB"));

        vboxLayout4->addWidget(oversamplingCB);

        exportLoopCB = new QCheckBox(qualityGroupBox);
        exportLoopCB->setObjectName(QString::fromUtf8("exportLoopCB"));

        vboxLayout4->addWidget(exportLoopCB);

        renderMarkersCB = new QCheckBox(qualityGroupBox);
        renderMarkersCB->setObjectName(QString::fromUtf8("renderMarkersCB"));

        vboxLayout4->addWidget(renderMarkersCB);

        spacerItem1 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout4->addItem(spacerItem1);


        hboxLayout->addWidget(qualityGroupBox);


        vboxLayout->addLayout(hboxLayout);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        spacerItem2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem2);

        startButton = new QPushButton(ExportProjectDialog);
        startButton->setObjectName(QString::fromUtf8("startButton"));

        hboxLayout1->addWidget(startButton);

        cancelButton = new QPushButton(ExportProjectDialog);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));

        hboxLayout1->addWidget(cancelButton);


        vboxLayout->addLayout(hboxLayout1);

        progressBar = new QProgressBar(ExportProjectDialog);
        progressBar->setObjectName(QString::fromUtf8("progressBar"));
        progressBar->setEnabled(false);
        progressBar->setValue(0);

        vboxLayout->addWidget(progressBar);


        retranslateUi(ExportProjectDialog);
        QObject::connect(cancelButton, SIGNAL(clicked()), ExportProjectDialog, SLOT(reject()));
        QObject::connect(fileFormatCB, SIGNAL(currentIndexChanged(int)), ExportProjectDialog, SLOT(onFileFormatChanged(int)));

        depthCB->setCurrentIndex(1);
        stereoModeComboBox->setCurrentIndex(1);
        bitrateCB->setCurrentIndex(2);
        interpolationCB->setCurrentIndex(2);


        QMetaObject::connectSlotsByName(ExportProjectDialog);
    } // setupUi

    void retranslateUi(QDialog *ExportProjectDialog)
    {
        ExportProjectDialog->setWindowTitle(QCoreApplication::translate("ExportProjectDialog", "Export project", nullptr));
        outputGroupBox->setTitle(QCoreApplication::translate("ExportProjectDialog", "Output", nullptr));
        label->setText(QCoreApplication::translate("ExportProjectDialog", "File format:", nullptr));
        labelSampleRate->setText(QCoreApplication::translate("ExportProjectDialog", "Samplerate:", nullptr));
        samplerateCB->setItemText(0, QCoreApplication::translate("ExportProjectDialog", "44100 Hz", nullptr));
        samplerateCB->setItemText(1, QCoreApplication::translate("ExportProjectDialog", "48000 Hz", nullptr));
        samplerateCB->setItemText(2, QCoreApplication::translate("ExportProjectDialog", "88200 Hz", nullptr));
        samplerateCB->setItemText(3, QCoreApplication::translate("ExportProjectDialog", "96000 Hz", nullptr));
        samplerateCB->setItemText(4, QCoreApplication::translate("ExportProjectDialog", "192000 Hz", nullptr));

        labelBitDepth->setText(QCoreApplication::translate("ExportProjectDialog", "Depth:", nullptr));
        depthCB->setItemText(0, QCoreApplication::translate("ExportProjectDialog", "16 Bit Integer", nullptr));
        depthCB->setItemText(1, QCoreApplication::translate("ExportProjectDialog", "24 Bit Integer", nullptr));
        depthCB->setItemText(2, QCoreApplication::translate("ExportProjectDialog", "32 Bit Float", nullptr));

        labelStereoMode_3->setText(QCoreApplication::translate("ExportProjectDialog", "Stereo mode:", nullptr));
        stereoModeComboBox->setItemText(0, QCoreApplication::translate("ExportProjectDialog", "Stereo", nullptr));
        stereoModeComboBox->setItemText(1, QCoreApplication::translate("ExportProjectDialog", "Joint Stereo", nullptr));
        stereoModeComboBox->setItemText(2, QCoreApplication::translate("ExportProjectDialog", "Mono", nullptr));

        labelBitRate->setText(QCoreApplication::translate("ExportProjectDialog", "Bitrate:", nullptr));
        bitrateCB->setItemText(0, QCoreApplication::translate("ExportProjectDialog", "64 KBit/s", nullptr));
        bitrateCB->setItemText(1, QCoreApplication::translate("ExportProjectDialog", "128 KBit/s", nullptr));
        bitrateCB->setItemText(2, QCoreApplication::translate("ExportProjectDialog", "160 KBit/s", nullptr));
        bitrateCB->setItemText(3, QCoreApplication::translate("ExportProjectDialog", "192 KBit/s", nullptr));
        bitrateCB->setItemText(4, QCoreApplication::translate("ExportProjectDialog", "256 KBit/s", nullptr));
        bitrateCB->setItemText(5, QCoreApplication::translate("ExportProjectDialog", "320 KBit/s", nullptr));

        checkBoxVariableBitRate->setText(QCoreApplication::translate("ExportProjectDialog", "Use variable bitrate", nullptr));
        qualityGroupBox->setTitle(QCoreApplication::translate("ExportProjectDialog", "Quality settings", nullptr));
        label_2->setText(QCoreApplication::translate("ExportProjectDialog", "Interpolation:", nullptr));
        interpolationCB->setItemText(0, QCoreApplication::translate("ExportProjectDialog", "Zero Order Hold", nullptr));
        interpolationCB->setItemText(1, QCoreApplication::translate("ExportProjectDialog", "Sinc Fastest", nullptr));
        interpolationCB->setItemText(2, QCoreApplication::translate("ExportProjectDialog", "Sinc Medium (recommended)", nullptr));
        interpolationCB->setItemText(3, QCoreApplication::translate("ExportProjectDialog", "Sinc Best (very slow!)", nullptr));

        label_4->setText(QCoreApplication::translate("ExportProjectDialog", "Oversampling (use with care!):", nullptr));
        oversamplingCB->setItemText(0, QCoreApplication::translate("ExportProjectDialog", "1x (None)", nullptr));
        oversamplingCB->setItemText(1, QCoreApplication::translate("ExportProjectDialog", "2x", nullptr));
        oversamplingCB->setItemText(2, QCoreApplication::translate("ExportProjectDialog", "4x", nullptr));
        oversamplingCB->setItemText(3, QCoreApplication::translate("ExportProjectDialog", "8x", nullptr));

        exportLoopCB->setText(QCoreApplication::translate("ExportProjectDialog", "Export as loop (remove end silence)", nullptr));
        renderMarkersCB->setText(QCoreApplication::translate("ExportProjectDialog", "Export between loop markers", nullptr));
        startButton->setText(QCoreApplication::translate("ExportProjectDialog", "Start", nullptr));
        cancelButton->setText(QCoreApplication::translate("ExportProjectDialog", "Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ExportProjectDialog: public Ui_ExportProjectDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EXPORT_PROJECT_H
