/****************************************************************************
** Meta object code from reading C++ file 'PianoRoll.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../lmms/include/PianoRoll.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PianoRoll.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PianoRoll_t {
    QByteArrayData data[54];
    char stringdata0[707];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PianoRoll_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PianoRoll_t qt_meta_stringdata_PianoRoll = {
    {
QT_MOC_LITERAL(0, 0, 9), // "PianoRoll"
QT_MOC_LITERAL(1, 10, 21), // "currentPatternChanged"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 33), // "semiToneMarkerMenuScaleSetEna..."
QT_MOC_LITERAL(4, 67, 33), // "semiToneMarkerMenuChordSetEna..."
QT_MOC_LITERAL(5, 101, 15), // "positionChanged"
QT_MOC_LITERAL(6, 117, 8), // "MidiTime"
QT_MOC_LITERAL(7, 126, 4), // "play"
QT_MOC_LITERAL(8, 131, 6), // "record"
QT_MOC_LITERAL(9, 138, 15), // "recordAccompany"
QT_MOC_LITERAL(10, 154, 4), // "stop"
QT_MOC_LITERAL(11, 159, 15), // "startRecordNote"
QT_MOC_LITERAL(12, 175, 4), // "Note"
QT_MOC_LITERAL(13, 180, 1), // "n"
QT_MOC_LITERAL(14, 182, 16), // "finishRecordNote"
QT_MOC_LITERAL(15, 199, 11), // "horScrolled"
QT_MOC_LITERAL(16, 211, 7), // "new_pos"
QT_MOC_LITERAL(17, 219, 11), // "verScrolled"
QT_MOC_LITERAL(18, 231, 11), // "setEditMode"
QT_MOC_LITERAL(19, 243, 4), // "mode"
QT_MOC_LITERAL(20, 248, 17), // "copySelectedNotes"
QT_MOC_LITERAL(21, 266, 16), // "cutSelectedNotes"
QT_MOC_LITERAL(22, 283, 10), // "pasteNotes"
QT_MOC_LITERAL(23, 294, 19), // "deleteSelectedNotes"
QT_MOC_LITERAL(24, 314, 14), // "updatePosition"
QT_MOC_LITERAL(25, 329, 1), // "t"
QT_MOC_LITERAL(26, 331, 23), // "updatePositionAccompany"
QT_MOC_LITERAL(27, 355, 14), // "zoomingChanged"
QT_MOC_LITERAL(28, 370, 15), // "quantizeChanged"
QT_MOC_LITERAL(29, 386, 13), // "quantizeNotes"
QT_MOC_LITERAL(30, 400, 24), // "updateSemiToneMarkerMenu"
QT_MOC_LITERAL(31, 425, 18), // "changeNoteEditMode"
QT_MOC_LITERAL(32, 444, 1), // "i"
QT_MOC_LITERAL(33, 446, 12), // "markSemiTone"
QT_MOC_LITERAL(34, 459, 11), // "hidePattern"
QT_MOC_LITERAL(35, 471, 8), // "Pattern*"
QT_MOC_LITERAL(36, 480, 7), // "pattern"
QT_MOC_LITERAL(37, 488, 22), // "selectRegionFromPixels"
QT_MOC_LITERAL(38, 511, 6), // "xStart"
QT_MOC_LITERAL(39, 518, 4), // "xEnd"
QT_MOC_LITERAL(40, 523, 12), // "barLineColor"
QT_MOC_LITERAL(41, 536, 13), // "beatLineColor"
QT_MOC_LITERAL(42, 550, 9), // "lineColor"
QT_MOC_LITERAL(43, 560, 13), // "noteModeColor"
QT_MOC_LITERAL(44, 574, 9), // "noteColor"
QT_MOC_LITERAL(45, 584, 8), // "barColor"
QT_MOC_LITERAL(46, 593, 17), // "selectedNoteColor"
QT_MOC_LITERAL(47, 611, 9), // "textColor"
QT_MOC_LITERAL(48, 621, 14), // "textColorLight"
QT_MOC_LITERAL(49, 636, 10), // "textShadow"
QT_MOC_LITERAL(50, 647, 19), // "markedSemitoneColor"
QT_MOC_LITERAL(51, 667, 11), // "noteOpacity"
QT_MOC_LITERAL(52, 679, 11), // "noteBorders"
QT_MOC_LITERAL(53, 691, 15) // "backgroundShade"

    },
    "PianoRoll\0currentPatternChanged\0\0"
    "semiToneMarkerMenuScaleSetEnabled\0"
    "semiToneMarkerMenuChordSetEnabled\0"
    "positionChanged\0MidiTime\0play\0record\0"
    "recordAccompany\0stop\0startRecordNote\0"
    "Note\0n\0finishRecordNote\0horScrolled\0"
    "new_pos\0verScrolled\0setEditMode\0mode\0"
    "copySelectedNotes\0cutSelectedNotes\0"
    "pasteNotes\0deleteSelectedNotes\0"
    "updatePosition\0t\0updatePositionAccompany\0"
    "zoomingChanged\0quantizeChanged\0"
    "quantizeNotes\0updateSemiToneMarkerMenu\0"
    "changeNoteEditMode\0i\0markSemiTone\0"
    "hidePattern\0Pattern*\0pattern\0"
    "selectRegionFromPixels\0xStart\0xEnd\0"
    "barLineColor\0beatLineColor\0lineColor\0"
    "noteModeColor\0noteColor\0barColor\0"
    "selectedNoteColor\0textColor\0textColorLight\0"
    "textShadow\0markedSemitoneColor\0"
    "noteOpacity\0noteBorders\0backgroundShade"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PianoRoll[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      27,   14, // methods
      14,  206, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  149,    2, 0x06 /* Public */,
       3,    1,  150,    2, 0x06 /* Public */,
       4,    1,  153,    2, 0x06 /* Public */,
       5,    1,  156,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    0,  159,    2, 0x09 /* Protected */,
       8,    0,  160,    2, 0x09 /* Protected */,
       9,    0,  161,    2, 0x09 /* Protected */,
      10,    0,  162,    2, 0x09 /* Protected */,
      11,    1,  163,    2, 0x09 /* Protected */,
      14,    1,  166,    2, 0x09 /* Protected */,
      15,    1,  169,    2, 0x09 /* Protected */,
      17,    1,  172,    2, 0x09 /* Protected */,
      18,    1,  175,    2, 0x09 /* Protected */,
      20,    0,  178,    2, 0x09 /* Protected */,
      21,    0,  179,    2, 0x09 /* Protected */,
      22,    0,  180,    2, 0x09 /* Protected */,
      23,    0,  181,    2, 0x09 /* Protected */,
      24,    1,  182,    2, 0x09 /* Protected */,
      26,    1,  185,    2, 0x09 /* Protected */,
      27,    0,  188,    2, 0x09 /* Protected */,
      28,    0,  189,    2, 0x09 /* Protected */,
      29,    0,  190,    2, 0x09 /* Protected */,
      30,    0,  191,    2, 0x09 /* Protected */,
      31,    1,  192,    2, 0x09 /* Protected */,
      33,    1,  195,    2, 0x09 /* Protected */,
      34,    1,  198,    2, 0x09 /* Protected */,
      37,    2,  201,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, 0x80000000 | 6,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, QMetaType::Int,   16,
    QMetaType::Void, QMetaType::Int,   16,
    QMetaType::Void, QMetaType::Int,   19,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 6,   25,
    QMetaType::Void, 0x80000000 | 6,   25,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   32,
    QMetaType::Void, QMetaType::Int,   32,
    QMetaType::Void, 0x80000000 | 35,   36,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   38,   39,

 // properties: name, type, flags
      40, QMetaType::QColor, 0x00095103,
      41, QMetaType::QColor, 0x00095103,
      42, QMetaType::QColor, 0x00095103,
      43, QMetaType::QColor, 0x00095103,
      44, QMetaType::QColor, 0x00095103,
      45, QMetaType::QColor, 0x00095103,
      46, QMetaType::QColor, 0x00095103,
      47, QMetaType::QColor, 0x00095103,
      48, QMetaType::QColor, 0x00095103,
      49, QMetaType::QColor, 0x00095103,
      50, QMetaType::QColor, 0x00095103,
      51, QMetaType::Int, 0x00095103,
      52, QMetaType::Bool, 0x00095103,
      53, QMetaType::QColor, 0x00095103,

       0        // eod
};

void PianoRoll::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PianoRoll *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->currentPatternChanged(); break;
        case 1: _t->semiToneMarkerMenuScaleSetEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->semiToneMarkerMenuChordSetEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->positionChanged((*reinterpret_cast< const MidiTime(*)>(_a[1]))); break;
        case 4: _t->play(); break;
        case 5: _t->record(); break;
        case 6: _t->recordAccompany(); break;
        case 7: _t->stop(); break;
        case 8: _t->startRecordNote((*reinterpret_cast< const Note(*)>(_a[1]))); break;
        case 9: _t->finishRecordNote((*reinterpret_cast< const Note(*)>(_a[1]))); break;
        case 10: _t->horScrolled((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->verScrolled((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->setEditMode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->copySelectedNotes(); break;
        case 14: _t->cutSelectedNotes(); break;
        case 15: _t->pasteNotes(); break;
        case 16: _t->deleteSelectedNotes(); break;
        case 17: _t->updatePosition((*reinterpret_cast< const MidiTime(*)>(_a[1]))); break;
        case 18: _t->updatePositionAccompany((*reinterpret_cast< const MidiTime(*)>(_a[1]))); break;
        case 19: _t->zoomingChanged(); break;
        case 20: _t->quantizeChanged(); break;
        case 21: _t->quantizeNotes(); break;
        case 22: _t->updateSemiToneMarkerMenu(); break;
        case 23: _t->changeNoteEditMode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 24: _t->markSemiTone((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 25: _t->hidePattern((*reinterpret_cast< Pattern*(*)>(_a[1]))); break;
        case 26: _t->selectRegionFromPixels((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PianoRoll::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PianoRoll::currentPatternChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PianoRoll::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PianoRoll::semiToneMarkerMenuScaleSetEnabled)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (PianoRoll::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PianoRoll::semiToneMarkerMenuChordSetEnabled)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (PianoRoll::*)(const MidiTime & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PianoRoll::positionChanged)) {
                *result = 3;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<PianoRoll *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QColor*>(_v) = _t->barLineColor(); break;
        case 1: *reinterpret_cast< QColor*>(_v) = _t->beatLineColor(); break;
        case 2: *reinterpret_cast< QColor*>(_v) = _t->lineColor(); break;
        case 3: *reinterpret_cast< QColor*>(_v) = _t->noteModeColor(); break;
        case 4: *reinterpret_cast< QColor*>(_v) = _t->noteColor(); break;
        case 5: *reinterpret_cast< QColor*>(_v) = _t->barColor(); break;
        case 6: *reinterpret_cast< QColor*>(_v) = _t->selectedNoteColor(); break;
        case 7: *reinterpret_cast< QColor*>(_v) = _t->textColor(); break;
        case 8: *reinterpret_cast< QColor*>(_v) = _t->textColorLight(); break;
        case 9: *reinterpret_cast< QColor*>(_v) = _t->textShadow(); break;
        case 10: *reinterpret_cast< QColor*>(_v) = _t->markedSemitoneColor(); break;
        case 11: *reinterpret_cast< int*>(_v) = _t->noteOpacity(); break;
        case 12: *reinterpret_cast< bool*>(_v) = _t->noteBorders(); break;
        case 13: *reinterpret_cast< QColor*>(_v) = _t->backgroundShade(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<PianoRoll *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setBarLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 1: _t->setBeatLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 2: _t->setLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 3: _t->setNoteModeColor(*reinterpret_cast< QColor*>(_v)); break;
        case 4: _t->setNoteColor(*reinterpret_cast< QColor*>(_v)); break;
        case 5: _t->setBarColor(*reinterpret_cast< QColor*>(_v)); break;
        case 6: _t->setSelectedNoteColor(*reinterpret_cast< QColor*>(_v)); break;
        case 7: _t->setTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 8: _t->setTextColorLight(*reinterpret_cast< QColor*>(_v)); break;
        case 9: _t->setTextShadow(*reinterpret_cast< QColor*>(_v)); break;
        case 10: _t->setMarkedSemitoneColor(*reinterpret_cast< QColor*>(_v)); break;
        case 11: _t->setNoteOpacity(*reinterpret_cast< int*>(_v)); break;
        case 12: _t->setNoteBorders(*reinterpret_cast< bool*>(_v)); break;
        case 13: _t->setBackgroundShade(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject PianoRoll::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_PianoRoll.data,
    qt_meta_data_PianoRoll,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PianoRoll::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PianoRoll::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PianoRoll.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int PianoRoll::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 27;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 14;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 14;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 14;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 14;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 14;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void PianoRoll::currentPatternChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void PianoRoll::semiToneMarkerMenuScaleSetEnabled(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void PianoRoll::semiToneMarkerMenuChordSetEnabled(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void PianoRoll::positionChanged(const MidiTime & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
struct qt_meta_stringdata_PianoRollWindow_t {
    QByteArrayData data[4];
    char stringdata0[54];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PianoRollWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PianoRollWindow_t qt_meta_stringdata_PianoRollWindow = {
    {
QT_MOC_LITERAL(0, 0, 15), // "PianoRollWindow"
QT_MOC_LITERAL(1, 16, 21), // "currentPatternChanged"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 14) // "patternRenamed"

    },
    "PianoRollWindow\0currentPatternChanged\0"
    "\0patternRenamed"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PianoRollWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   25,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void PianoRollWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PianoRollWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->currentPatternChanged(); break;
        case 1: _t->patternRenamed(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PianoRollWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PianoRollWindow::currentPatternChanged)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject PianoRollWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<Editor::staticMetaObject>(),
    qt_meta_stringdata_PianoRollWindow.data,
    qt_meta_data_PianoRollWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PianoRollWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PianoRollWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PianoRollWindow.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "SerializingObject"))
        return static_cast< SerializingObject*>(this);
    return Editor::qt_metacast(_clname);
}

int PianoRollWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Editor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void PianoRollWindow::currentPatternChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
