/****************************************************************************
** Meta object code from reading C++ file 'MainWindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../lmms/include/MainWindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MainWindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MainWindow_t {
    QByteArrayData data[43];
    char stringdata0[646];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MainWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MainWindow_t qt_meta_stringdata_MainWindow = {
    {
QT_MOC_LITERAL(0, 0, 10), // "MainWindow"
QT_MOC_LITERAL(1, 11, 14), // "periodicUpdate"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 12), // "initProgress"
QT_MOC_LITERAL(4, 40, 3), // "msg"
QT_MOC_LITERAL(5, 44, 16), // "resetWindowTitle"
QT_MOC_LITERAL(6, 61, 9), // "emptySlot"
QT_MOC_LITERAL(7, 71, 18), // "enterWhatsThisMode"
QT_MOC_LITERAL(8, 90, 16), // "createNewProject"
QT_MOC_LITERAL(9, 107, 28), // "createNewProjectFromTemplate"
QT_MOC_LITERAL(10, 136, 8), // "QAction*"
QT_MOC_LITERAL(11, 145, 4), // "_idx"
QT_MOC_LITERAL(12, 150, 11), // "openProject"
QT_MOC_LITERAL(13, 162, 11), // "saveProject"
QT_MOC_LITERAL(14, 174, 13), // "saveProjectAs"
QT_MOC_LITERAL(15, 188, 23), // "saveProjectAsNewVersion"
QT_MOC_LITERAL(16, 212, 28), // "saveProjectAsDefaultTemplate"
QT_MOC_LITERAL(17, 241, 18), // "showSettingsDialog"
QT_MOC_LITERAL(18, 260, 9), // "aboutLMMS"
QT_MOC_LITERAL(19, 270, 4), // "help"
QT_MOC_LITERAL(20, 275, 25), // "toggleAutomationEditorWin"
QT_MOC_LITERAL(21, 301, 17), // "toggleBBEditorWin"
QT_MOC_LITERAL(22, 319, 9), // "forceShow"
QT_MOC_LITERAL(23, 329, 19), // "toggleSongEditorWin"
QT_MOC_LITERAL(24, 349, 21), // "toggleProjectNotesWin"
QT_MOC_LITERAL(25, 371, 16), // "toggleFxMixerWin"
QT_MOC_LITERAL(26, 388, 18), // "togglePianoRollWin"
QT_MOC_LITERAL(27, 407, 20), // "toggleControllerRack"
QT_MOC_LITERAL(28, 428, 20), // "updatePlayPauseIcons"
QT_MOC_LITERAL(29, 449, 21), // "updateUndoRedoButtons"
QT_MOC_LITERAL(30, 471, 4), // "undo"
QT_MOC_LITERAL(31, 476, 4), // "redo"
QT_MOC_LITERAL(32, 481, 8), // "autoSave"
QT_MOC_LITERAL(33, 490, 10), // "browseHelp"
QT_MOC_LITERAL(34, 501, 17), // "fillTemplatesMenu"
QT_MOC_LITERAL(35, 519, 25), // "openRecentlyOpenedProject"
QT_MOC_LITERAL(36, 545, 7), // "_action"
QT_MOC_LITERAL(37, 553, 8), // "showTool"
QT_MOC_LITERAL(38, 562, 32), // "updateRecentlyOpenedProjectsMenu"
QT_MOC_LITERAL(39, 595, 14), // "updateViewMenu"
QT_MOC_LITERAL(40, 610, 12), // "updateConfig"
QT_MOC_LITERAL(41, 623, 4), // "_who"
QT_MOC_LITERAL(42, 628, 17) // "onToggleMetronome"

    },
    "MainWindow\0periodicUpdate\0\0initProgress\0"
    "msg\0resetWindowTitle\0emptySlot\0"
    "enterWhatsThisMode\0createNewProject\0"
    "createNewProjectFromTemplate\0QAction*\0"
    "_idx\0openProject\0saveProject\0saveProjectAs\0"
    "saveProjectAsNewVersion\0"
    "saveProjectAsDefaultTemplate\0"
    "showSettingsDialog\0aboutLMMS\0help\0"
    "toggleAutomationEditorWin\0toggleBBEditorWin\0"
    "forceShow\0toggleSongEditorWin\0"
    "toggleProjectNotesWin\0toggleFxMixerWin\0"
    "togglePianoRollWin\0toggleControllerRack\0"
    "updatePlayPauseIcons\0updateUndoRedoButtons\0"
    "undo\0redo\0autoSave\0browseHelp\0"
    "fillTemplatesMenu\0openRecentlyOpenedProject\0"
    "_action\0showTool\0updateRecentlyOpenedProjectsMenu\0"
    "updateViewMenu\0updateConfig\0_who\0"
    "onToggleMetronome"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MainWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      36,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  194,    2, 0x06 /* Public */,
       3,    1,  195,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,  198,    2, 0x0a /* Public */,
       6,    0,  199,    2, 0x0a /* Public */,
       7,    0,  200,    2, 0x0a /* Public */,
       8,    0,  201,    2, 0x0a /* Public */,
       9,    1,  202,    2, 0x0a /* Public */,
      12,    0,  205,    2, 0x0a /* Public */,
      13,    0,  206,    2, 0x0a /* Public */,
      14,    0,  207,    2, 0x0a /* Public */,
      15,    0,  208,    2, 0x0a /* Public */,
      16,    0,  209,    2, 0x0a /* Public */,
      17,    0,  210,    2, 0x0a /* Public */,
      18,    0,  211,    2, 0x0a /* Public */,
      19,    0,  212,    2, 0x0a /* Public */,
      20,    0,  213,    2, 0x0a /* Public */,
      21,    1,  214,    2, 0x0a /* Public */,
      21,    0,  217,    2, 0x2a /* Public | MethodCloned */,
      23,    0,  218,    2, 0x0a /* Public */,
      24,    0,  219,    2, 0x0a /* Public */,
      25,    0,  220,    2, 0x0a /* Public */,
      26,    0,  221,    2, 0x0a /* Public */,
      27,    0,  222,    2, 0x0a /* Public */,
      28,    0,  223,    2, 0x0a /* Public */,
      29,    0,  224,    2, 0x0a /* Public */,
      30,    0,  225,    2, 0x0a /* Public */,
      31,    0,  226,    2, 0x0a /* Public */,
      32,    0,  227,    2, 0x0a /* Public */,
      33,    0,  228,    2, 0x08 /* Private */,
      34,    0,  229,    2, 0x08 /* Private */,
      35,    1,  230,    2, 0x08 /* Private */,
      37,    1,  233,    2, 0x08 /* Private */,
      38,    0,  236,    2, 0x08 /* Private */,
      39,    0,  237,    2, 0x08 /* Private */,
      40,    1,  238,    2, 0x08 /* Private */,
      42,    0,  241,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    4,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   22,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,   36,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,   41,
    QMetaType::Void,

       0        // eod
};

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MainWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->periodicUpdate(); break;
        case 1: _t->initProgress((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->resetWindowTitle(); break;
        case 3: _t->emptySlot(); break;
        case 4: _t->enterWhatsThisMode(); break;
        case 5: _t->createNewProject(); break;
        case 6: _t->createNewProjectFromTemplate((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 7: _t->openProject(); break;
        case 8: { bool _r = _t->saveProject();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->saveProjectAs();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->saveProjectAsNewVersion();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->saveProjectAsDefaultTemplate(); break;
        case 12: _t->showSettingsDialog(); break;
        case 13: _t->aboutLMMS(); break;
        case 14: _t->help(); break;
        case 15: _t->toggleAutomationEditorWin(); break;
        case 16: _t->toggleBBEditorWin((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 17: _t->toggleBBEditorWin(); break;
        case 18: _t->toggleSongEditorWin(); break;
        case 19: _t->toggleProjectNotesWin(); break;
        case 20: _t->toggleFxMixerWin(); break;
        case 21: _t->togglePianoRollWin(); break;
        case 22: _t->toggleControllerRack(); break;
        case 23: _t->updatePlayPauseIcons(); break;
        case 24: _t->updateUndoRedoButtons(); break;
        case 25: _t->undo(); break;
        case 26: _t->redo(); break;
        case 27: _t->autoSave(); break;
        case 28: _t->browseHelp(); break;
        case 29: _t->fillTemplatesMenu(); break;
        case 30: _t->openRecentlyOpenedProject((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 31: _t->showTool((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 32: _t->updateRecentlyOpenedProjectsMenu(); break;
        case 33: _t->updateViewMenu(); break;
        case 34: _t->updateConfig((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 35: _t->onToggleMetronome(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MainWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MainWindow::periodicUpdate)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (MainWindow::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MainWindow::initProgress)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_MainWindow.data,
    qt_meta_data_MainWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MainWindow.stringdata0))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 36;
    }
    return _id;
}

// SIGNAL 0
void MainWindow::periodicUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MainWindow::initProgress(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
