/****************************************************************************
** Meta object code from reading C++ file 'SongEditor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../lmms/include/SongEditor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SongEditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SongEditor_t {
    QByteArrayData data[31];
    char stringdata0[425];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SongEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SongEditor_t qt_meta_stringdata_SongEditor = {
    {
QT_MOC_LITERAL(0, 0, 10), // "SongEditor"
QT_MOC_LITERAL(1, 11, 8), // "scrolled"
QT_MOC_LITERAL(2, 20, 0), // ""
QT_MOC_LITERAL(3, 21, 7), // "new_pos"
QT_MOC_LITERAL(4, 29, 22), // "selectRegionFromPixels"
QT_MOC_LITERAL(5, 52, 6), // "xStart"
QT_MOC_LITERAL(6, 59, 4), // "xEnd"
QT_MOC_LITERAL(7, 64, 16), // "stopSelectRegion"
QT_MOC_LITERAL(8, 81, 16), // "updateRubberband"
QT_MOC_LITERAL(9, 98, 11), // "setEditMode"
QT_MOC_LITERAL(10, 110, 8), // "EditMode"
QT_MOC_LITERAL(11, 119, 4), // "mode"
QT_MOC_LITERAL(12, 124, 15), // "setEditModeDraw"
QT_MOC_LITERAL(13, 140, 17), // "setEditModeSelect"
QT_MOC_LITERAL(14, 158, 14), // "updatePosition"
QT_MOC_LITERAL(15, 173, 8), // "MidiTime"
QT_MOC_LITERAL(16, 182, 1), // "t"
QT_MOC_LITERAL(17, 184, 18), // "updatePositionLine"
QT_MOC_LITERAL(18, 203, 14), // "setHighQuality"
QT_MOC_LITERAL(19, 218, 15), // "setMasterVolume"
QT_MOC_LITERAL(20, 234, 7), // "new_val"
QT_MOC_LITERAL(21, 242, 21), // "showMasterVolumeFloat"
QT_MOC_LITERAL(22, 264, 23), // "updateMasterVolumeFloat"
QT_MOC_LITERAL(23, 288, 21), // "hideMasterVolumeFloat"
QT_MOC_LITERAL(24, 310, 14), // "setMasterPitch"
QT_MOC_LITERAL(25, 325, 20), // "showMasterPitchFloat"
QT_MOC_LITERAL(26, 346, 22), // "updateMasterPitchFloat"
QT_MOC_LITERAL(27, 369, 20), // "hideMasterPitchFloat"
QT_MOC_LITERAL(28, 390, 15), // "updateScrollBar"
QT_MOC_LITERAL(29, 406, 3), // "len"
QT_MOC_LITERAL(30, 410, 14) // "zoomingChanged"

    },
    "SongEditor\0scrolled\0\0new_pos\0"
    "selectRegionFromPixels\0xStart\0xEnd\0"
    "stopSelectRegion\0updateRubberband\0"
    "setEditMode\0EditMode\0mode\0setEditModeDraw\0"
    "setEditModeSelect\0updatePosition\0"
    "MidiTime\0t\0updatePositionLine\0"
    "setHighQuality\0setMasterVolume\0new_val\0"
    "showMasterVolumeFloat\0updateMasterVolumeFloat\0"
    "hideMasterVolumeFloat\0setMasterPitch\0"
    "showMasterPitchFloat\0updateMasterPitchFloat\0"
    "hideMasterPitchFloat\0updateScrollBar\0"
    "len\0zoomingChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SongEditor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      20,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  114,    2, 0x0a /* Public */,
       4,    2,  117,    2, 0x0a /* Public */,
       7,    0,  122,    2, 0x0a /* Public */,
       8,    0,  123,    2, 0x0a /* Public */,
       9,    1,  124,    2, 0x0a /* Public */,
      12,    0,  127,    2, 0x0a /* Public */,
      13,    0,  128,    2, 0x0a /* Public */,
      14,    1,  129,    2, 0x0a /* Public */,
      17,    0,  132,    2, 0x0a /* Public */,
      18,    1,  133,    2, 0x08 /* Private */,
      19,    1,  136,    2, 0x08 /* Private */,
      21,    0,  139,    2, 0x08 /* Private */,
      22,    1,  140,    2, 0x08 /* Private */,
      23,    0,  143,    2, 0x08 /* Private */,
      24,    1,  144,    2, 0x08 /* Private */,
      25,    0,  147,    2, 0x08 /* Private */,
      26,    1,  148,    2, 0x08 /* Private */,
      27,    0,  151,    2, 0x08 /* Private */,
      28,    1,  152,    2, 0x08 /* Private */,
      30,    0,  155,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    5,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 15,   16,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   29,
    QMetaType::Void,

       0        // eod
};

void SongEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SongEditor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->scrolled((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->selectRegionFromPixels((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 2: _t->stopSelectRegion(); break;
        case 3: _t->updateRubberband(); break;
        case 4: _t->setEditMode((*reinterpret_cast< EditMode(*)>(_a[1]))); break;
        case 5: _t->setEditModeDraw(); break;
        case 6: _t->setEditModeSelect(); break;
        case 7: _t->updatePosition((*reinterpret_cast< const MidiTime(*)>(_a[1]))); break;
        case 8: _t->updatePositionLine(); break;
        case 9: _t->setHighQuality((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->setMasterVolume((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->showMasterVolumeFloat(); break;
        case 12: _t->updateMasterVolumeFloat((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->hideMasterVolumeFloat(); break;
        case 14: _t->setMasterPitch((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->showMasterPitchFloat(); break;
        case 16: _t->updateMasterPitchFloat((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->hideMasterPitchFloat(); break;
        case 18: _t->updateScrollBar((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 19: _t->zoomingChanged(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SongEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<TrackContainerView::staticMetaObject>(),
    qt_meta_stringdata_SongEditor.data,
    qt_meta_data_SongEditor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SongEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SongEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SongEditor.stringdata0))
        return static_cast<void*>(this);
    return TrackContainerView::qt_metacast(_clname);
}

int SongEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TrackContainerView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 20;
    }
    return _id;
}
struct qt_meta_stringdata_SongEditorWindow_t {
    QByteArrayData data[9];
    char stringdata0[98];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SongEditorWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SongEditorWindow_t qt_meta_stringdata_SongEditorWindow = {
    {
QT_MOC_LITERAL(0, 0, 16), // "SongEditorWindow"
QT_MOC_LITERAL(1, 17, 13), // "playTriggered"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 7), // "resized"
QT_MOC_LITERAL(4, 40, 4), // "play"
QT_MOC_LITERAL(5, 45, 6), // "record"
QT_MOC_LITERAL(6, 52, 15), // "recordAccompany"
QT_MOC_LITERAL(7, 68, 4), // "stop"
QT_MOC_LITERAL(8, 73, 24) // "adjustUiAfterProjectLoad"

    },
    "SongEditorWindow\0playTriggered\0\0resized\0"
    "play\0record\0recordAccompany\0stop\0"
    "adjustUiAfterProjectLoad"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SongEditorWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x06 /* Public */,
       3,    0,   50,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   51,    2, 0x09 /* Protected */,
       5,    0,   52,    2, 0x09 /* Protected */,
       6,    0,   53,    2, 0x09 /* Protected */,
       7,    0,   54,    2, 0x09 /* Protected */,
       8,    0,   55,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void SongEditorWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SongEditorWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->playTriggered(); break;
        case 1: _t->resized(); break;
        case 2: _t->play(); break;
        case 3: _t->record(); break;
        case 4: _t->recordAccompany(); break;
        case 5: _t->stop(); break;
        case 6: _t->adjustUiAfterProjectLoad(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SongEditorWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SongEditorWindow::playTriggered)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (SongEditorWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SongEditorWindow::resized)) {
                *result = 1;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SongEditorWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<Editor::staticMetaObject>(),
    qt_meta_stringdata_SongEditorWindow.data,
    qt_meta_data_SongEditorWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SongEditorWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SongEditorWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SongEditorWindow.stringdata0))
        return static_cast<void*>(this);
    return Editor::qt_metacast(_clname);
}

int SongEditorWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Editor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void SongEditorWindow::playTriggered()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void SongEditorWindow::resized()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
