// This file is autogenerated. Changes will be overwritten.
#include "DGKZTCOIDE/moc_ActionGroup.cpp"
#include "DGKZTCOIDE/moc_AudioAlsaSetupWidget.cpp"
#include "DGKZTCOIDE/moc_AudioDeviceSetupWidget.cpp"
#include "DGKZTCOIDE/moc_AudioJack.cpp"
#include "DGKZTCOIDE/moc_AudioPortAudio.cpp"
#include "DGKZTCOIDE/moc_AudioSoundIo.cpp"
#include "DGKZTCOIDE/moc_AutomatableButton.cpp"
#include "DGKZTCOIDE/moc_AutomatableModel.cpp"
#include "DGKZTCOIDE/moc_AutomatableModelView.cpp"
#include "DGKZTCOIDE/moc_AutomatableSlider.cpp"
#include "DGKZTCOIDE/moc_AutomationEditor.cpp"
#include "DGKZTCOIDE/moc_AutomationPattern.cpp"
#include "DGKZTCOIDE/moc_AutomationPatternView.cpp"
#include "DGKZTCOIDE/moc_AutomationTrack.cpp"
#include "DGKZTCOIDE/moc_BBEditor.cpp"
#include "DGKZTCOIDE/moc_BBTrack.cpp"
#include "DGKZTCOIDE/moc_BBTrackContainer.cpp"
#include "DGKZTCOIDE/moc_CPULoadWidget.cpp"
#include "DGKZTCOIDE/moc_CaptionMenu.cpp"
#include "DGKZTCOIDE/moc_ComboBox.cpp"
#include "DGKZTCOIDE/moc_ComboBoxModel.cpp"
#include "DGKZTCOIDE/moc_ConfigManager.cpp"
#include "DGKZTCOIDE/moc_Controller.cpp"
#include "DGKZTCOIDE/moc_ControllerConnection.cpp"
#include "DGKZTCOIDE/moc_ControllerConnectionDialog.cpp"
#include "DGKZTCOIDE/moc_ControllerDialog.cpp"
#include "DGKZTCOIDE/moc_ControllerRackView.cpp"
#include "DGKZTCOIDE/moc_ControllerView.cpp"
#include "DGKZTCOIDE/moc_DetuningHelper.cpp"
#include "DGKZTCOIDE/moc_DummyEffect.cpp"
#include "DGKZTCOIDE/moc_Editor.cpp"
#include "DGKZTCOIDE/moc_Effect.cpp"
#include "DGKZTCOIDE/moc_EffectChain.cpp"
#include "DGKZTCOIDE/moc_EffectControlDialog.cpp"
#include "DGKZTCOIDE/moc_EffectRackView.cpp"
#include "DGKZTCOIDE/moc_EffectSelectDialog.cpp"
#include "DGKZTCOIDE/moc_EffectView.cpp"
#include "DGKZTCOIDE/moc_Engine.cpp"
#include "DGKZTCOIDE/moc_EnvelopeAndLfoParameters.cpp"
#include "DGKZTCOIDE/moc_EnvelopeAndLfoView.cpp"
#include "DGKZTCOIDE/moc_ExportProjectDialog.cpp"
#include "DGKZTCOIDE/moc_FadeButton.cpp"
#include "DGKZTCOIDE/moc_Fader.cpp"
#include "DGKZTCOIDE/moc_FileBrowser.cpp"
#include "DGKZTCOIDE/moc_FileDialog.cpp"
#include "DGKZTCOIDE/moc_FxLine.cpp"
#include "DGKZTCOIDE/moc_FxMixer.cpp"
#include "DGKZTCOIDE/moc_FxMixerView.cpp"
#include "DGKZTCOIDE/moc_Graph.cpp"
#include "DGKZTCOIDE/moc_GroupBox.cpp"
#include "DGKZTCOIDE/moc_GuiApplication.cpp"
#include "DGKZTCOIDE/moc_InstrumentFunctionViews.cpp"
#include "DGKZTCOIDE/moc_InstrumentFunctions.cpp"
#include "DGKZTCOIDE/moc_InstrumentMidiIOView.cpp"
#include "DGKZTCOIDE/moc_InstrumentSoundShaping.cpp"
#include "DGKZTCOIDE/moc_InstrumentSoundShapingView.cpp"
#include "DGKZTCOIDE/moc_InstrumentTrack.cpp"
#include "DGKZTCOIDE/moc_Knob.cpp"
#include "DGKZTCOIDE/moc_LadspaControl.cpp"
#include "DGKZTCOIDE/moc_LadspaControlView.cpp"
#include "DGKZTCOIDE/moc_LcdSpinBox.cpp"
#include "DGKZTCOIDE/moc_LcdWidget.cpp"
#include "DGKZTCOIDE/moc_LedCheckbox.cpp"
#include "DGKZTCOIDE/moc_LeftRightNav.cpp"
#include "DGKZTCOIDE/moc_LfoController.cpp"
#include "DGKZTCOIDE/moc_LmmsPalette.cpp"
#include "DGKZTCOIDE/moc_MainWindow.cpp"
#include "DGKZTCOIDE/moc_MeterDialog.cpp"
#include "DGKZTCOIDE/moc_MeterModel.cpp"
#include "DGKZTCOIDE/moc_MidiAlsaSeq.cpp"
#include "DGKZTCOIDE/moc_MidiApple.cpp"
#include "DGKZTCOIDE/moc_MidiController.cpp"
#include "DGKZTCOIDE/moc_MidiJack.cpp"
#include "DGKZTCOIDE/moc_MidiPort.cpp"
#include "DGKZTCOIDE/moc_MidiPortMenu.cpp"
#include "DGKZTCOIDE/moc_MidiSetupWidget.cpp"
#include "DGKZTCOIDE/moc_MidiWinMM.cpp"
#include "DGKZTCOIDE/moc_Mixer.cpp"
#include "DGKZTCOIDE/moc_Model.cpp"
#include "DGKZTCOIDE/moc_NStateButton.cpp"
#include "DGKZTCOIDE/moc_Pattern.cpp"
#include "DGKZTCOIDE/moc_PeakController.cpp"
#include "DGKZTCOIDE/moc_PianoRoll.cpp"
#include "DGKZTCOIDE/moc_PianoView.cpp"
#include "DGKZTCOIDE/moc_PixmapButton.cpp"
#include "DGKZTCOIDE/moc_Plugin.cpp"
#include "DGKZTCOIDE/moc_PluginBrowser.cpp"
#include "DGKZTCOIDE/moc_ProjectNotes.cpp"
#include "DGKZTCOIDE/moc_ProjectRenderer.cpp"
#include "DGKZTCOIDE/moc_RemotePlugin.cpp"
#include "DGKZTCOIDE/moc_RenameDialog.cpp"
#include "DGKZTCOIDE/moc_RenderManager.cpp"
#include "DGKZTCOIDE/moc_RingBuffer.cpp"
#include "DGKZTCOIDE/moc_RowTableView.cpp"
#include "DGKZTCOIDE/moc_Rubberband.cpp"
#include "DGKZTCOIDE/moc_SampleBuffer.cpp"
#include "DGKZTCOIDE/moc_SampleTrack.cpp"
#include "DGKZTCOIDE/moc_SetupDialog.cpp"
#include "DGKZTCOIDE/moc_SideBar.cpp"
#include "DGKZTCOIDE/moc_SideBarWidget.cpp"
#include "DGKZTCOIDE/moc_Song.cpp"
#include "DGKZTCOIDE/moc_SongEditor.cpp"
#include "DGKZTCOIDE/moc_SubWindow.cpp"
#include "DGKZTCOIDE/moc_TabBar.cpp"
#include "DGKZTCOIDE/moc_TabButton.cpp"
#include "DGKZTCOIDE/moc_TabWidget.cpp"
#include "DGKZTCOIDE/moc_TempoSyncKnob.cpp"
#include "DGKZTCOIDE/moc_TempoSyncKnobModel.cpp"
#include "DGKZTCOIDE/moc_TextFloat.cpp"
#include "DGKZTCOIDE/moc_TimeDisplayWidget.cpp"
#include "DGKZTCOIDE/moc_TimeLineWidget.cpp"
#include "DGKZTCOIDE/moc_ToolButton.cpp"
#include "DGKZTCOIDE/moc_Track.cpp"
#include "DGKZTCOIDE/moc_TrackContainer.cpp"
#include "DGKZTCOIDE/moc_TrackContainerView.cpp"
#include "DGKZTCOIDE/moc_TrackLabelButton.cpp"
#include "DGKZTCOIDE/moc_TrackRenameLineEdit.cpp"
#include "DGKZTCOIDE/moc_VersionedSaveDialog.cpp"
#include "DGKZTCOIDE/moc_VisualizationWidget.cpp"
#include "DGKZTCOIDE/moc_VstSyncController.cpp"
